@@ -75,16 +75,17 @@ class vmmConsole(gobject.GObject):
         self.gtk_settings_accel = None
 
         self.vncViewer = gtkvnc.Display()
-        self.window.get_widget("console-vnc-align").add(self.vncViewer)
+        self.window.get_widget("console-vnc-viewport").add(self.vncViewer)
         self.vncViewer.realize()
         self.vncTunnel = None
         if self.config.get_console_keygrab() == 2:
             self.vncViewer.set_keyboard_grab(True)
-            self.vncViewer.set_pointer_grab(True)
         else:
             self.vncViewer.set_keyboard_grab(False)
-            self.vncViewer.set_pointer_grab(False)
-        self.vncViewer.set_pointer_local(True)
+        self.vncViewer.set_pointer_grab(True)
+        if not topwin.is_composited():
+            self.vncViewer.set_scaling(True)
+            self.window.get_widget("menu-view-scale-display").set_active(True)
 
         self.vncViewer.connect("vnc-pointer-grab", self.notify_grabbed)
         self.vncViewer.connect("vnc-pointer-ungrab", self.notify_ungrabbed)
@@ -147,6 +148,8 @@ class vmmConsole(gobject.GObject):
             "on_menu_send_caf7_activate": self.send_key,
             "on_menu_send_caf8_activate": self.send_key,
             "on_menu_send_printscreen_activate": self.send_key,
+
+            "on_menu_view_scale_display_activate": self.scale_display,
             })
 
         self.vm.connect("status-changed", self.update_widget_states)
@@ -165,8 +168,10 @@ class vmmConsole(gobject.GObject):
     # widget it still seems to show scrollbars. So we do evil stuff here
     def _force_resize(self, src, size):
         w,h = src.get_size_request()
+        if w == -1 or h == -1:
+            return
+
         self.window.get_widget("console-screenshot").set_size_request(w, h)
-        self.window.get_widget("console-vnc-scroll").set_size_request(w, h)
         topw,toph = self.window.get_widget("vmm-console").size_request()
 
         padx = topw-w
@@ -178,6 +183,7 @@ class vmmConsole(gobject.GObject):
         maxh = rooth - 100 - pady
         self.window.get_widget("console-vnc-viewport").set_size_request(w, h)
         self.window.get_widget("console-screenshot-viewport").set_size_request(w, h)
+        self.window.get_widget("console-vnc-scroll").set_size_request(w, h)
         if w > maxw or h > maxh:
             self.window.get_widget("console-vnc-scroll").set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
             self.window.get_widget("console-screenshot-scroll").set_policy(gtk.POLICY_ALWAYS, gtk.POLICY_ALWAYS)
@@ -185,22 +191,6 @@ class vmmConsole(gobject.GObject):
             self.window.get_widget("console-vnc-scroll").set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
             self.window.get_widget("console-screenshot-scroll").set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
 
-    # Auto-increase the window size to fit the console - within reason
-    # though, cos we don't want a min window size greater than the screen
-    # the user has scrollbars anyway if they want it smaller / it can't fit
-    def autosize(self, src, size):
-        rootWidth = gtk.gdk.screen_width()
-        rootHeight = gtk.gdk.screen_height()
-
-        vncWidth, vncHeight = src.get_size_request()
-
-        if vncWidth > (rootWidth-200):
-            vncWidth = rootWidth - 200
-        if vncHeight > (rootHeight-200):
-            vncHeight = rootHeight - 200
-
-        self.window.get_widget("console-vnc-vp").set_size_request(vncWidth+2, vncHeight+2)
-
     def send_key(self, src):
         keys = None
         if src.get_name() == "menu-send-cad":
@@ -287,10 +277,14 @@ class vmmConsole(gobject.GObject):
     def keygrab_changed(self, src, ignore1=None,ignore2=None,ignore3=None):
         if self.config.get_console_keygrab() == 2:
             self.vncViewer.set_keyboard_grab(True)
-            self.vncViewer.set_pointer_grab(True)
         else:
             self.vncViewer.set_keyboard_grab(False)
-            self.vncViewer.set_pointer_grab(False)
+
+    def scale_display(self, src):
+        if src.get_active():
+            self.vncViewer.set_scaling(True)
+        else:
+            self.vncViewer.set_scaling(False)
 
     def toggle_fullscreen(self, src):
         if src.get_active():
@@ -393,7 +387,7 @@ class vmmConsole(gobject.GObject):
         finally:
             gtk.gdk.threads_leave()
 
-    def open_tunnel(self, server, vncaddr ,vncport):
+    def open_tunnel(self, server, vncaddr, vncport, username):
         if self.vncTunnel is not None:
             return
 
@@ -407,7 +401,11 @@ class vmmConsole(gobject.GObject):
             os.close(1)
             os.dup(fds[1].fileno())
             os.dup(fds[1].fileno())
-            os.execlp("ssh", "ssh", "-p", "22", "-l", "root", server, "nc", vncaddr, str(vncport))
+            argv = ["ssh", "ssh", "-p", "22"] 
+            if username:
+                argv += ['-l', username]
+            argv += [ server, "nc", vncaddr, str(vncport) ]
+            os.execlp(*argv)
             os._exit(1)
         else:
             fds[1].close()
@@ -433,14 +431,18 @@ class vmmConsole(gobject.GObject):
 
         logging.debug("Trying console login")
         password = self.window.get_widget("console-auth-password").get_text()
-        protocol, host, port, trans = self.vm.get_graphics_console()
+        protocol, host, port, trans, username = self.vm.get_graphics_console()
 
         if protocol is None:
             logging.debug("No graphics configured in guest")
             self.activate_unavailable_page(_("Console not configured for guest"))
             return
 
-        uri = str(protocol) + "://" + str(host) + ":" + str(port)
+        uri = str(protocol) + "://"
+        if username:
+            uri = uri + str(username) + '@'
+        uri = uri + str(host) + ":" + str(port)
+
         logging.debug("Graphics console configured at " + uri)
 
         if protocol != "vnc":
@@ -457,7 +459,7 @@ class vmmConsole(gobject.GObject):
         logging.debug("Starting connect process for %s %s" % (host, str(port)))
         try:
             if trans is not None and trans in ("ssh", "ext"):
-                fd = self.open_tunnel(host, "127.0.0.1", port)
+                fd = self.open_tunnel(host, "127.0.0.1", port, username)
                 self.vncViewer.open_fd(fd)
             else:
                 self.vncViewer.open_host(host, str(port))
@@ -661,6 +663,8 @@ class vmmConsole(gobject.GObject):
                     cr.show_text(overlay)
                     screenshot.set_from_pixmap(pixmap, None)
                     self.activate_screenshot_page()
+                elif self.window.get_widget("console-pages").get_current_page() == PAGE_SCREENSHOT:
+                    pass
                 else:
                     if self.window.get_widget("console-pages").get_current_page() != PAGE_UNAVAILABLE:
                         self.vncViewer.close()
