@@ -404,10 +404,11 @@ class vmmCreateNetwork(gobject.GObject):
         # Find info about all current present media
         for d in self.hal_iface.FindDeviceByCapability("volume"):
             vol = self.bus.get_object("org.freedesktop.Hal", d)
-            if vol.GetPropertyBoolean("volume.is_disc") and \
-                   vol.GetPropertyBoolean("volume.disc.has_data"):
-                devnode = vol.GetProperty("block.device")
-                label = vol.GetProperty("volume.label")
+            volif = dbus.Interface(vol, "org.freedesktop.Hal.Device")
+            if volif.GetPropertyBoolean("volume.is_disc") and \
+                   volif.GetPropertyBoolean("volume.disc.has_data"):
+                devnode = volif.GetProperty("block.device")
+                label = volif.GetProperty("volume.label")
                 if label == None or len(label) == 0:
                     label = devnode
                 vollabel[devnode] = label
@@ -416,7 +417,8 @@ class vmmCreateNetwork(gobject.GObject):
 
         for d in self.hal_iface.FindDeviceByCapability("storage.cdrom"):
             dev = self.bus.get_object("org.freedesktop.Hal", d)
-            devnode = dev.GetProperty("block.device")
+            devif = dbus.Interface(dev, "org.freedesktop.Hal.Device")
+            devnode = devif.GetProperty("block.device")
             if vollabel.has_key(devnode):
                 model.append([devnode, vollabel[devnode], True, volpath[devnode]])
             else:
