@@ -49,6 +49,7 @@ HW_LIST_TYPE_DISK = 2
 HW_LIST_TYPE_NIC = 3
 HW_LIST_TYPE_INPUT = 4
 HW_LIST_TYPE_GRAPHICS = 5
+HW_LIST_TYPE_BOOT = 6
 
 class vmmDetails(gobject.GObject):
     __gsignals__ = {
@@ -134,6 +135,9 @@ class vmmDetails(gobject.GObject):
             "on_config_memory_changed": self.config_memory_changed,
             "on_config_maxmem_changed": self.config_maxmem_changed,
             "on_config_memory_apply_clicked": self.config_memory_apply,
+            "on_config_boot_device_changed": self.config_boot_options_changed,
+            "on_config_autostart_changed": self.config_boot_options_changed,
+            "on_config_boot_apply_clicked": self.config_boot_options_apply,
             "on_details_help_activate": self.show_help,
 
             "on_config_cdrom_connect_clicked": self.toggle_cdrom,
@@ -216,6 +220,9 @@ class vmmDetails(gobject.GObject):
                 self.refresh_input_page()
             elif pagetype == HW_LIST_TYPE_GRAPHICS:
                 self.refresh_graphics_page()
+            elif pagetype == HW_LIST_TYPE_BOOT:
+                self.refresh_boot_page()
+                self.window.get_widget("config-boot-options-apply").set_sensitive(False)
             else:
                 pagenum = -1
 
@@ -298,6 +305,7 @@ class vmmDetails(gobject.GObject):
                     self.window.get_widget("details-menu-pause").set_active(False)
         except:
             self.ignorePause = False
+            raise
         self.ignorePause = False
 
         self.window.get_widget("overview-status-text").set_text(self.vm.run_status())
@@ -495,6 +503,40 @@ class vmmDetails(gobject.GObject):
             else:
                 self.window.get_widget("config-input-remove").set_sensitive(True)
 
+    def refresh_boot_page(self):
+        # Refresh autostart
+        try:
+            autoval = self.vm.get_autostart()
+            self.window.get_widget("config-autostart").set_active(autoval)
+            self.window.get_widget("config-autostart").set_sensitive(True)
+        except libvirt.libvirtError, e:
+            # Autostart isn't supported
+            self.window.get_widget("config-autostart").set_active(False)
+            self.window.get_widget("config-autostart").set_sensitive(False)
+
+        # Refresh Boot Device list and correct selection
+        boot_combo = self.window.get_widget("config-boot-device")
+        if not self.vm.is_hvm():
+            # Boot dev selection not supported for PV guest
+            boot_combo.set_sensitive(False)
+            boot_combo.set_active(-1)
+            return
+
+        self.repopulate_boot_list()
+        bootdev = self.vm.get_boot_device()
+        boot_combo = self.window.get_widget("config-boot-device")
+        boot_model = boot_combo.get_model()
+        for i in range(0, len(boot_model)):
+            if bootdev == boot_model[i][2]:
+                boot_combo.set_active(i)
+                break
+
+        if boot_model[0][2] == None:
+            # If no boot devices, select the 'No Device' entry
+            boot_combo.set_active(0)
+
+        # TODO: if nothing selected, what to select? auto change device?
+
     def config_vcpus_changed(self, src):
         self.window.get_widget("config-vcpus-apply").set_sensitive(True)
 
@@ -518,15 +560,44 @@ class vmmDetails(gobject.GObject):
     def config_memory_apply(self, src):
         status = self.vm.status()
         if status in [ libvirt.VIR_DOMAIN_SHUTOFF, libvirt.VIR_DOMAIN_CRASHED ]:
-            memory = self.window.get_widget("config-maxmem").get_adjustment().value
-            logging.info("Setting max memory for " + self.vm.get_uuid() + " to " + str(memory))
-            self.vm.set_max_memory(memory*1024)
-        memory = self.window.get_widget("config-memory").get_adjustment().value
-        logging.info("Setting memory for " + self.vm.get_uuid() + " to " + str(memory))
-        self.vm.set_memory(memory*1024)
+            maxmem = self.window.get_widget("config-maxmem").get_adjustment()
+        curmem = self.window.get_widget("config-memory").get_adjustment()
+        if maxmem.value < curmem.value:
+            logging.error("Failed to set max memory " + str(maxmem.value) + \
+                          " below current memory " + str(curmem.value))
+            maxmem.value = curmem.value
+        # set max memory allocated for domain
+        self.vm.set_max_memory(maxmem.value*1024)
+        # set currently allocated memory
+        self.vm.set_memory(curmem.value*1024)
+        logging.info("Setting max-memory for " + self.vm.get_uuid() + \
+                     " to " + str(maxmem.value))
+        logging.info("Setting memory for " + self.vm.get_uuid() + \
+                     " to " + str(curmem.value))
 
         self.window.get_widget("config-memory-apply").set_sensitive(False)
 
+    def config_boot_options_changed(self, src):
+        self.window.get_widget("config-boot-options-apply").set_sensitive(True)
+
+    def config_boot_options_apply(self, src):
+        boot = self.window.get_widget("config-boot-device")
+        auto = self.window.get_widget("config-autostart")
+        if auto.get_property("sensitive"):
+            try:
+                self.vm.set_autostart(auto.get_active())
+            except Exception, e:
+                self._err_dialog(_("Error changing autostart value: %s") % \
+                                 str(e), "".join(traceback.format_exc()))
+
+        if boot.get_property("sensitive"):
+            try:
+                self.vm.set_boot_device(boot.get_model()[boot.get_active()][2])
+                self.window.get_widget("config-boot-options-apply").set_sensitive(False)
+            except Exception, e:
+                self._err_dialog(_("Error changing boot device: %s" % str(e)),
+                                 "".join(traceback.format_exc()))
+                return
 
     def remove_disk(self, src):
         vmlist = self.window.get_widget("hw-list")
@@ -597,14 +668,30 @@ class vmmDetails(gobject.GObject):
         hwCol.add_attribute(hw_img, 'stock-size', HW_LIST_COL_STOCK_SIZE)
         hwCol.add_attribute(hw_img, 'pixbuf', HW_LIST_COL_PIXBUF)
         self.window.get_widget("hw-list").append_column(hwCol)
+        self.prepare_boot_list()
 
         self.populate_hw_list()
+        self.repopulate_boot_list()
+
+    def prepare_boot_list(self):
+        boot_list = self.window.get_widget("config-boot-device")
+        # model = [ display name, icon name, boot type (hd, fd, etc) ]
+        boot_list_model = gtk.ListStore(str, str, str)
+        boot_list.set_model(boot_list_model)
+
+        icon = gtk.CellRendererPixbuf()
+        boot_list.pack_start(icon, False)
+        boot_list.add_attribute(icon, 'stock-id', 1)
+        text = gtk.CellRendererText()
+        boot_list.pack_start(text, True)
+        boot_list.add_attribute(text, 'text', 0)
 
     def populate_hw_list(self):
         hw_list_model = self.window.get_widget("hw-list").get_model()
         hw_list_model.clear()
         hw_list_model.append(["Processor", None, 0, self.pixbuf_processor, HW_LIST_TYPE_CPU, []])
         hw_list_model.append(["Memory", None, 0, self.pixbuf_memory, HW_LIST_TYPE_MEMORY, []])
+        hw_list_model.append(["Boot Options", None, 0, self.pixbuf_memory, HW_LIST_TYPE_BOOT, []])
         self.repopulate_hw_list()
 
     def repopulate_hw_list(self):
@@ -725,6 +812,36 @@ class vmmDetails(gobject.GObject):
                 # Now actually remove it
                 hw_list_model.remove(iter)
 
+    def repopulate_boot_list(self):
+        hw_list_model = self.window.get_widget("hw-list").get_model()
+        boot_combo = self.window.get_widget("config-boot-device")
+        boot_model = boot_combo.get_model()
+        boot_model.clear()
+        found_dev = {}
+        for row in hw_list_model:
+            if row[4] == HW_LIST_TYPE_DISK:
+                disk = row[5]
+                if disk[2] == virtinst.VirtualDisk.DEVICE_DISK and not \
+                   found_dev.get(virtinst.VirtualDisk.DEVICE_DISK, False):
+                    boot_model.append(["Hard Disk", gtk.STOCK_HARDDISK, "hd"])
+                    found_dev[virtinst.VirtualDisk.DEVICE_DISK] = True
+                elif disk[2] == virtinst.VirtualDisk.DEVICE_CDROM and not \
+                     found_dev.get(virtinst.VirtualDisk.DEVICE_CDROM, False):
+                    boot_model.append(["CDROM", gtk.STOCK_CDROM, "cdrom"])
+                    found_dev[virtinst.VirtualDisk.DEVICE_CDROM] = True
+                elif disk[2] == virtinst.VirtualDisk.DEVICE_FLOPPY and not \
+                     found_dev.get(virtinst.VirtualDisk.DEVICE_FLOPPY, False):
+                    boot_model.append(["Floppy", gtk.STOCK_FLOPPY, "fd"])
+                    found_dev[virtinst.VirtualDisk.DEVICE_FLOPPY] = True
+            elif row[4] == HW_LIST_TYPE_NIC and not \
+                 found_dev.get(HW_LIST_TYPE_NIC, False):
+                boot_model.append(["Network (PXE)", gtk.STOCK_NETWORK, "network"])
+                found_dev[HW_LIST_TYPE_NIC] = True
+
+        if len(boot_model) <= 0:
+            boot_model.append([_("No Boot Device"), None, None])
+
+        boot_combo.set_model(boot_model)
 
     def add_hardware(self, src):
         if self.addhw is None:
